@@ -63,227 +63,20 @@ https://github.com/traversc/qs
 #include <Rversion.h>
 
 #include "RApiSerializeAPI.h"
+#include "sf_external.h"
+
 #include "zstd.h"
 #include "lz4.h"
 #include "lz4hc.h"
 #include "BLOSC/shuffle_routines.h"
 #include "BLOSC/unshuffle_routines.h"
 
-#include "xxhash/xxhash.c" // static linking
+#include "xxhash/xxhash.c"
 #include <R_ext/Rdynload.h>
 
-// do not include by default due to CRAN policies
-// import based on configure script
-// #ifdef USE_R_CONNECTION
-// #define class aclass
-// #define private aprivate
-// #include <R_ext/Connections.h>
-// #undef class
-// #undef private
-// 
-// #if defined(R_VERSION) && R_VERSION >= R_Version(3, 3, 0)
-// Rconnection r_get_connection(SEXP con) {
-//   return R_GetConnection(con);
-// }
-// #else
-// Rconnection r_get_connection(SEXP con) {
-//   if (!Rf_inherits(con, "connection"))
-//     Rcpp::stop("invalid connection");
-//   return getConnection(Rf_asInteger(con));
-// }
-// #endif
-// #endif
 
 using namespace Rcpp;
 
-////////////////////////////////////////////////////////////////
-// alt rep string class
-////////////////////////////////////////////////////////////////
-#if R_VERSION >= R_Version(3, 5, 0)
-#define ALTREP_SUPPORTED
-
-// load alt-rep header -- see altrepisode package by Romain Francois
-#if R_VERSION < R_Version(3, 6, 0)
-#define class klass
-extern "C" {
-#include <R_ext/Altrep.h>
-}
-#undef class
-#else
-#include <R_ext/Altrep.h>
-#endif
-
-
-struct stdvec_data {
-  std::vector<std::string> strings;
-  std::vector<uint8_t> encodings;
-  R_xlen_t vec_size; 
-  stdvec_data(uint64_t N) {
-    strings = std::vector<std::string>(N);
-    encodings = std::vector<uint8_t>(N);
-    vec_size = N;
-  }
-};
-
-// instead of defining a set of free functions, we structure them
-// together in a struct
-struct stdvec_string {
-  static R_altrep_class_t class_t;
-  static SEXP Make(stdvec_data* data, bool owner){
-    SEXP xp = PROTECT(R_MakeExternalPtr(data, R_NilValue, R_NilValue));
-    if (owner) {
-      R_RegisterCFinalizerEx(xp, stdvec_string::Finalize, TRUE);
-    }
-    SEXP res = R_new_altrep(class_t, xp, R_NilValue);
-    UNPROTECT(1);
-    return res;
-  }
-  
-  // finalizer for the external pointer
-  static void Finalize(SEXP xp){
-    delete static_cast<stdvec_data*>(R_ExternalPtrAddr(xp));
-  }
-  
-  // get the std::vector<string>* from the altrep object `x`
-  static stdvec_data* Ptr(SEXP vec) {
-    return static_cast<stdvec_data*>(R_ExternalPtrAddr(R_altrep_data1(vec)));
-  }
-  
-  // same, but as a reference, for convenience
-  static stdvec_data& Get(SEXP vec) {
-    return *static_cast<stdvec_data*>(R_ExternalPtrAddr(R_altrep_data1(vec)));
-  }
-  
-  // ALTREP methods -------------------
-  
-  // The length of the object
-  static R_xlen_t Length(SEXP vec){
-    return Get(vec).vec_size;
-  }
-  
-  // What gets printed when .Internal(inspect()) is used
-  static Rboolean Inspect(SEXP x, int pre, int deep, int pvec, void (*inspect_subtree)(SEXP, int, int, int)){
-    Rprintf("qs alt-rep stdvec_string (len=%d, ptr=%p)\n", Length(x), Ptr(x));
-    return TRUE;
-  }
-  
-  // ALTVEC methods ------------------
-  static SEXP Materialize(SEXP vec) {
-    SEXP data2 = R_altrep_data2(vec);
-    if (data2 != R_NilValue) {
-      return data2;
-    }
-    R_xlen_t n = Length(vec);
-    data2 = PROTECT(Rf_allocVector(STRSXP, n));
-    
-    auto data1 = Get(vec);
-    for (R_xlen_t i = 0; i < n; i++) {
-      switch(data1.encodings[i]) {
-      case 1:
-        SET_STRING_ELT(data2, i, Rf_mkCharLenCE(data1.strings[i].data(), data1.strings[i].size(), CE_NATIVE) );
-        break;
-      case 2:
-        SET_STRING_ELT(data2, i, Rf_mkCharLenCE(data1.strings[i].data(), data1.strings[i].size(), CE_UTF8) );
-        break;
-      case 3:
-        SET_STRING_ELT(data2, i, Rf_mkCharLenCE(data1.strings[i].data(), data1.strings[i].size(), CE_LATIN1) );
-        break;
-      case 4:
-        SET_STRING_ELT(data2, i, Rf_mkCharLenCE(data1.strings[i].data(), data1.strings[i].size(), CE_BYTES) );
-        break;
-      default:
-        SET_STRING_ELT(data2, i, NA_STRING);
-      break;
-      }
-    }
-    
-    // free up some memory -- shrink to fit is a non-binding request
-    data1.encodings.resize(0);
-    data1.encodings.shrink_to_fit();
-    data1.strings.resize(0);
-    data1.strings.shrink_to_fit();
-    
-    R_set_altrep_data2(vec, data2);
-    UNPROTECT(1);
-    return data2;
-  }
-  
-  // The start of the data, i.e. the underlying double* array from the std::vector<double>
-  // This is guaranteed to never allocate (in the R sense)
-  static const void* Dataptr_or_null(SEXP vec) {
-    SEXP data2 = R_altrep_data2(vec);
-    if (data2 == R_NilValue) return nullptr;
-    return STDVEC_DATAPTR(data2);
-  }
-  
-  // same in this case, writeable is ignored
-  static void* Dataptr(SEXP vec, Rboolean writeable) {
-    return STDVEC_DATAPTR(Materialize(vec));
-  }
-  
-  
-  // ALTSTRING methods -----------------
-  // the element at the index `i`
-  // this does not do bounds checking because that's expensive, so
-  // the caller must take care of that
-  static SEXP string_Elt(SEXP vec, R_xlen_t i){
-    SEXP data2 = Materialize(vec);
-    return STRING_ELT(data2, i);
-    // switch(data1.encodings[i]) {
-    // case 1:
-    //   return Rf_mkCharLenCE(data1.strings[i].data(), data1.strings[i].size(), CE_NATIVE);
-    // case 2:
-    //   return Rf_mkCharLenCE(data1.strings[i].data(), data1.strings[i].size(), CE_UTF8);
-    // case 3:
-    //   return Rf_mkCharLenCE(data1.strings[i].data(), data1.strings[i].size(), CE_LATIN1);
-    // case 4:
-    //   return Rf_mkCharLenCE(data1.strings[i].data(), data1.strings[i].size(), CE_BYTES);
-    // default:
-    //   return NA_STRING;
-    // break;
-    // }
-  }
-  
-  static void string_Set_elt(SEXP vec, R_xlen_t i, SEXP new_val) {
-    SEXP data2 = Materialize(vec);
-    SET_STRING_ELT(data2, i, new_val);
-  }
-  
-  // -------- initialize the altrep class with the methods above
-  
-  static void Init(DllInfo* dll){
-    class_t = R_make_altstring_class("stdvec_string", "qs", dll);
-    
-    // altrep
-    R_set_altrep_Length_method(class_t, Length);
-    R_set_altrep_Inspect_method(class_t, Inspect);
-    
-    // altvec
-    R_set_altvec_Dataptr_method(class_t, Dataptr);
-    R_set_altvec_Dataptr_or_null_method(class_t, Dataptr_or_null);
-    
-    // altstring
-    R_set_altstring_Elt_method(class_t, string_Elt);
-    R_set_altstring_Set_elt_method(class_t, string_Set_elt);
-  }
-  
-};
-
-// static initialization of stdvec_double::class_t
-R_altrep_class_t stdvec_string::class_t;
-
-// Called the package is loaded (needs Rcpp 0.12.18.3)
-// [[Rcpp::init]]
-void init_stdvec_double(DllInfo* dll){
-  stdvec_string::Init(dll);
-}
-
-#else
-void init_stdvec_double(DllInfo* dll) {
-  // void;
-  // do nothing -- need a dummy function becasue of Rcpp attributes
-}
-#endif
 ////////////////////////////////////////////////////////////////
 // common utility functions and constants
 ////////////////////////////////////////////////////////////////
